#include "image.h"
using namespace stdi;

int  stdi::round_double(double x) 
{
 int    ix;
 double dx;

 ix = (int) x;
 dx = (double) x - (double) ix;

 if (dx >= 0.5) return(ix+1);
 else return(ix);
}

void stdi::out_of_memory() {
  cerr << "Error: out of memory" << endl;
  exit(1);
}

ArrayData::ArrayData(): data(0), count(1) {}
ArrayData::~ArrayData() {delete []data; }
void ArrayData::ref() { count++; }
bool ArrayData::deref() { return !--count; }

Image::Image()
{
  memset(this, 0, sizeof(Image));
}

Image::Image(int _n_x, int _n_y, int _n_z, int _n_c, int _n_t )
  : n_x(std::max(0,_n_x)), n_y(std::max(0, _n_y)), n_zval(std::max(0,_n_z)), n_c(std::max(0,_n_c)), n_t(std::max(0,_n_t)),
    n_xy(_n_x*_n_y), n_pix(_n_x*_n_y*_n_z*_n_c*_n_t), n_xyz(_n_x*_n_y*_n_z), n_xyzc(_n_x*_n_y*_n_z*_n_c), n_z(_n_z*_n_c*_n_t)
{  
    
} 

Image::~Image() { }



bool Image::isNull() const { return n_pix==0; }
bool Image::equalSize(const Image &a) const
{
  return n_x==a.nx() && n_y==a.ny()&&n_z==a.nz();
}

vector<bool> Image::interior() const 
{ return Image::interior(n_x, n_y, n_z); }
    
vector<bool> Image::interior(int n_x, int n_y, int n_z)
{
  const int n_pix = n_x*n_y*n_z;
  
  if (n_pix <= 0) return  vector<bool>();
  vector<bool> mask(n_pix, true);        
  const int n_xy = n_x*n_y ;
  int x, y, z, pp, qq;    
  if (n_z == 1) { // 2D    
    for(y = 0, pp = 0, qq = n_x-1; y < n_y; y++, pp += n_x, qq += n_x) {
      mask[pp] = false; mask[qq] = false;
    }    
    for(x = 0, pp = (n_y-1)*n_x; x < n_x; x++, pp++) {
      mask[x] = false; mask[pp] = false;
    }
    
  }  else { // 3D
    for(y = 0, pp = 0,  qq = n_xy*(n_z-1); y < n_y; y++)
      for(x = 0; x < n_x; x++, pp++, qq++) {
	mask[pp] = false;
	mask[qq] = false;
      }
    pp = (n_y-1)*n_x;
      for(z = 0; z < n_z; z++)
	for(x = 0; x < n_x; x++) {
	  mask[qq = z*n_xy + x] = false;
	  mask[pp + qq] = false;
	}      
      pp = n_x - 1;
      for(z = 0; z < n_z; z++)
	for(y = 0; y < n_y; y++) {
	  mask[qq = z*n_xy + y*n_x] = false;
	  mask[pp + qq] = false;
	}    
  }    
  return mask;
}
  

Image *Image::reset(Image *&p1, Image *p2)
{
  if (p1==p2) return p1;
  delete p1;
  p1 = p2;
  return p1;
}


Roi::Roi(int x_min, int y_min, int x_max, int y_max)
  : _xmin(x_min), _ymin(y_min), _xmax(x_max), _ymax(y_max)
{
}


Roi::Roi()
  : _xmin(0), _ymin(0), _xmax(-1), _ymax(-1)
{
}

Roi::~Roi()
{
}

    
void Roi::set(int x_min, int y_min, int x_max, int y_max) 
{
  _xmin = x_min;
  _ymin = y_min;
  
  _xmax = x_max;
  _ymax = y_max;
}

Roi::coord_type Roi::xmin() const
{
  return _xmin;
}

Roi::coord_type Roi::ymin() const
{
  return _ymin;
}

Roi::coord_type Roi::xmax() const
{
  return _xmax;
}

Roi::coord_type Roi::ymax() const
{
  return _ymax;
}


Image *Roi::getSubImage(const Image &I) const 
{
  return I.roi( Point3D(_xmin, _ymin, 0), 
		Point3D(_xmax, _ymax, 0) );
}


Image *Roi::getSubImage(const Image &I, const Roi &r)
{
  return r.getSubImage(I);
}


Image *Roi::getSubImage(const Image &I, 
			int x_min, int y_min, int x_max, int y_max)
{
  return I.roi( Point3D(x_min, y_min, 0), 
		Point3D(x_max, y_max, 0) );
}


Roi::Roi(const PointArray &b)
  : _xmin(0), _ymin(0), _xmax(-1), _ymax(-1)
{
  if (b.empty())
    return;
  
  _xmin = _xmax = b[0].x();
  _ymin = _ymax = b[0].y();
  
  for(int i=1; i<b.size(); i++) {
    coord_type x = b[i].x();
    _xmin = min(_xmin, x);
    _xmax = max(_xmax, x);
    
    coord_type y = b[i].y();
    _ymin = min(_ymin, y);
    _ymax = max(_ymax, y);

  }
}


bool Roi::isEmpty() const
{
  return _xmax<_xmin || _ymax<_ymin;
}


FloatImage stdi::toFloatImage(const Image &I)
{
  if (I.isNull())
    return FloatImage();

  FloatImage J(I.nx(), I.ny(), I.nzval(), I.nc(), I.nt() );
  float *p = J.ptr();
  const int npix = I.npix();

  PixelFormat pf = I.format();
  
  if (pf==BYTE) {
    byte *q = (byte*)I.data();
    for(int i=0; i<npix; i++)
      p[i] = q[i];
    return J;
  }
  
  
  if (pf==SHORT) {
    short *q = (short*)I.data();
    for(int i=0; i<npix; i++)
      p[i] = q[i];
    return J;
  }
  
  if (pf==INT) {
    int *q = (int*)I.data();
    for(int i=0; i<npix; i++)
      p[i] = q[i];
    return J;
  }

  if (pf==FLOAT) {
    float *q = (float*)I.data();
    memcpy(p, q, npix*sizeof(float));
    return J;
  }
  if (pf==USHORT) {   //sree
     unsigned short *q = (unsigned short *)I.data();
    for(int i=0; i<npix; i++)
      p[i] = q[i];
    return J;
  }
  if (pf==UINT) {   //sree
     unsigned int *q = (unsigned int *)I.data();
    for(int i=0; i<npix; i++)
      p[i] = q[i];
    return J;
  }
  if( pf == RGB ){
      for( int i = 0; i < npix; i ++ )
	  p[i] = I.pixel(i);
      return J;
  }
  if( pf == INTRGB ){
      for( int i = 0; i < npix; i ++ )
	  p[i] = I.pixel(i);
      return J;
  }
  if( pf == SHORTRGB ){
      for( int i = 0; i < npix; i ++ )
	  p[i] = I.pixel(i);
      return J;
  }
  if( pf == FLOATRGB ){
      for( int i = 0; i < npix; i ++ )
	  p[i] = I.pixel(i);
      return J;
  }
  return FloatImage();
}

ByteImage stdi::toByteImage(const Image &img, bool forced_scaling)
{
    if (img.isNull()) return ByteImage();
    if (img.format()==BYTE && !forced_scaling) 
	return * ( (ByteImage *) &img );
    const int npix ( img.npix() );
    ByteImage a(img.nx(), img.ny(), img.nz());
    
    float val = img.pixel(0);
    float minv = val;
    float maxv = val;
    for(int k=1; k<npix; k++) {
	val = img.pixel(k); 
	if (val<minv) minv = val;
	else if (val>maxv) maxv = val;
    }
    if (fabs(minv-maxv)<0.0001) return a;
    
    if (minv>=0 && maxv<=255 && !forced_scaling)
	for(int k=0; k<npix; k++)
	    // a.set(k, byte(max(0, min(255, (int)img.pixel(k)))) );
	    a.set(k, byte( (int)img.pixel(k) ));
    else
	for(int k=0; k<npix; k++)
	    a.set(k, 255*(img.pixel(k)-minv)/(maxv-minv));
    
    return a;
}

RgbImage stdi::toRgbImage(const Image &I)
{
    if (I.isNull())
	return RgbImage();
    
    RgbImage J(I.nx(), I.ny(), I.nzval(), I.nc(), I.nt() );
    Rgb *p = J.ptr();
    const int npix = I.npix();
    
    PixelFormat pf = I.format();
        
    
    if( pf == BYTE || pf == SHORT || pf == INT || pf == FLOAT || pf == USHORT || pf == UINT ){

	ByteImage a = toByteImage(I,true);
	byte *q = (byte*)a.data();
	for(int i = 0; i < npix; i++ )
	    p[i] = Rgb(q[i],q[i],q[i]);
	return J;
    }
    
    if( pf == RGB ){
	
	Rgb * q = (Rgb*)I.data();
	memcpy(p, q, npix*sizeof(Rgb));
	return J;
    }
    if( pf == RGBA ){
	Rgba * q = (Rgba*)I.data();
	for( int i = 0; i < npix; i ++ ){
	    float alpha = (float)q[i].alpha/255;
	    //cout << "alpha:\t"<<alpha<<endl;
	    byte r = q[i].red*alpha + (1-alpha)*255;
	    byte g = q[i].green*alpha + (1-alpha)*255;
	    byte b = q[i].blue*alpha + (1-alpha)*255;
	    p[i] = Rgb(r,g,b);
	}
	return J;
    }
   
    if( pf == INTRGB || pf == SHORTRGB || pf == FLOATRGB ){
	
	FloatImage grey(I.nx(),I.ny(),I.nz());
	float * pg = (float*)grey.data();
	for( int i = 0; i < npix; i ++ )
	    pg[i] = I.pixel(i);
	ByteImage bgrey = toByteImage(grey,true);
	
	if( pf == INTRGB ){
	    
	    IntRgb * q = (IntRgb*)I.data();
	    for( int i = 0; i < npix; i ++ ){
		float ratio = bgrey.at(i)/grey.at(i);
		p[i] = Rgb( q[i].red * ratio, q[i].green * ratio, q[i].blue * ratio );
	    }
	}
	if( pf == SHORTRGB ){
	    
	    ShortRgb * q = (ShortRgb*)I.data();
	    for( int i = 0; i < npix; i ++ ){
		float ratio = bgrey.at(i)/grey.at(i);
		p[i] = Rgb( q[i].red * ratio, q[i].green * ratio, q[i].blue * ratio );
	    }
	}
	if( pf == FLOATRGB ){
	    
	    FloatRgb * q = (FloatRgb*)I.data();
	    for( int i = 0; i < npix; i ++ ){
		float ratio = bgrey.at(i)/grey.at(i);
		p[i] = Rgb( q[i].red * ratio, q[i].green * ratio, q[i].blue * ratio );
	    }
	}
	return J;
    }
    return RgbImage();
}
