#include "circularvoting.h"
#include "image_io.h"
#include "image.h"
#include "bip.h"
#include "global.h"
#include <string>

using namespace std;
using namespace stdi;
using namespace voting;

int main( const int argc, const char ** argv ){
  
  if( argc != 9 ){
    cerr<<"Invalid number of parameters"<<endl;
    cerr<<"Usage:\t image_file_name channel hmin hmax radius min_grad min_vote sigma\n";
    return 1;
  }
  string fn = argv[1];
  int channel = atoi(argv[2]);
  float hmin = atof(argv[3]);
  float hmax = atof(argv[4]);
  int radius = atoi(argv[5]);
  float min_grad = atof(argv[6]);
  float min_vote = atof(argv[7]);
  float sigma = atof(argv[8]);
  
  Image * img = NULL;
  img = image_io::readImage(fn.c_str());

  if( img == NULL ){
    cerr<<"Image is not valid"<<endl;
    getchar();
    return 1;
  }
  
  cout << "channel:\t"<<channel<<endl;
  cout << "hmin:\t"<<hmin<<endl;
  cout << "hmax:\t"<<hmax<<endl;
  cout << "radius:\t"<<radius<<endl;
  cout << "min_grad:\t"<<min_grad<<endl;
  cout << "min_vote:\t"<<min_vote<<endl;
  cout << "sigma:\t"<<sigma<<endl;

  cout << "fmin:\t"<<img->fmin()<<endl;
  cout << "fmax:\t"<<img->fmax()<<endl;

  CircularVoting cv;
  cv.setParams(CircularVoting::BrightReg, hmin, hmax,radius, min_grad, min_vote, sigma, CircularVoting::PowMethod, (float) 1, false, true);
  cv.saveHistory(true);
  cv.setPrefix("tmp/");
  cv.compute(*(img->frame(channel)));
  PointArray seeds = cv.centers();
  vector<float> weights = cv.center_weights();

  int i;
  for( i = 0; i < seeds.size(); i ++ ){
    cout << "\nSeed#"<<i<<"\t("<<seeds[i].x()<<",\t"<<seeds[i].y()<<")\t";
    cout << "weights:\t"<<weights[i]<<endl;
  }
    
  delete img; img = NULL;

  return 0;
}
