/***********************************************************************
 ** point3d.h
 ** class Point3D: 3D point
 ***********************************************************************/

#if !defined(POINT3D_H)
#define POINT3D_H

#include <vector>

namespace stdi { 

  class  Point3D {
  
  public:

    // Typedefs
    typedef short coord_type;
  
    // Constructors
    Point3D(): xp(0), yp(0), zp(0) { }
    Point3D(int xpos, int ypos, int zpos) 
      :xp((coord_type)xpos), yp((coord_type)ypos), zp((coord_type)zpos) { }
    
    // Destructor
    ~Point3D(){}
      
    bool isNull() const { return xp==0 && yp==0 && zp==0; }

    int x() const { return xp; }    
    int y() const { return yp; }    
    int z() const { return zp; }    
    void setX( int x ) { xp = (coord_type)x; }    
    void setY( int y ) { yp = (coord_type)y; }    
    void setZ( int z ) { zp = (coord_type)z; }
    
    
    Point3D &operator+=( const Point3D &p );
    Point3D &operator-=( const Point3D &p );
    Point3D &operator*=( int c );
    Point3D &operator*=( double c );
    Point3D &operator/=( int c );
    Point3D &operator/=( double c );
    
    friend  bool   operator==( const Point3D &, const Point3D & );
    friend  bool  operator<( const Point3D &p1, const Point3D &p2 );
    friend  bool   operator!=( const Point3D &, const Point3D & );
    friend  const Point3D operator+( const Point3D &, const Point3D & );
    friend  const Point3D operator-( const Point3D &, const Point3D & );
    friend  const Point3D operator*( const Point3D &, int );
    friend  const Point3D operator*( int, const Point3D & );
    friend  const Point3D operator*( const Point3D &, double );
    friend  const Point3D operator*( double, const Point3D & );
    friend  const Point3D operator-( const Point3D & );
    friend  const Point3D operator/( const Point3D &, int );
    friend  const Point3D operator/( const Point3D &, double );

  private:
    
    coord_type xp, yp, zp;
    
  };
  
  typedef std::vector<Point3D> Point3DArray;

} // End of namespace stdi

#endif // POINT3D_H
