#include "point.h"
#include "point3d.h"
#include <iostream>
#include <cmath>
using namespace std;
using namespace stdi;

#define CHECK_MATH

namespace {

  void warningDivByZero()
  {
    cerr << "Warning: divided by 0" << endl;
    exit(1);
  }

}


Point &Point::operator+=( const Point &p )
{ xp+=p.xp; yp+=p.yp; return *this; }

Point &Point::operator-=( const Point &p )
{ xp-=p.xp; yp-=p.yp; return *this; }


// point + scalar, scalar + point
Point &Point::operator+=( int c )
{ xp+=(coord_type)c; yp+=(coord_type)c; return *this; }

Point &Point::operator+=( double c )
{ xp=(coord_type)(xp+c); yp=(coord_type)(yp+c); return *this; }

const Point stdi::operator+( const Point &p, int c )
{ return Point(p.xp+c, p.yp+c); }

const Point stdi::operator+( int c, const Point &p )
{ return Point(p.xp+c, p.yp+c); }

const Point stdi::operator+( const Point &p, double c )
{ return Point((Point::coord_type)(p.xp+c), (Point::coord_type)(p.yp+c)); }

const Point stdi::operator+( double c, const Point &p )
{ return Point((Point::coord_type)(p.xp+c), (Point::coord_type)(p.yp+c)); }


// point - scalar, scalar - point
Point &Point::operator-=( int c )
{ xp-=(coord_type)c; yp-=(coord_type)c; return *this; }

Point &Point::operator-=( double c )
{ xp=(coord_type)(xp-c); yp=(coord_type)(yp-c); return *this; }

const Point stdi::operator-( const Point &p, int c )
{ return Point(p.xp-c, p.yp-c); }

const Point stdi::operator-( int c, const Point &p )
{ return Point(p.xp-c, p.yp-c); }

const Point stdi::operator-( const Point &p, double c )
{ return Point((Point::coord_type)(p.xp-c), (Point::coord_type)(p.yp-c)); }

const Point stdi::operator-( double c, const Point &p )
{ return Point((Point::coord_type)(p.xp-c), (Point::coord_type)(p.yp-c)); }


// point * scalar, scalar * point
Point &Point::operator*=( int c )
{ xp*=(coord_type)c; yp*=(coord_type)c; return *this; }

Point &Point::operator*=( double c )
{ xp=(coord_type)(xp*c); yp=(coord_type)(yp*c); return *this; }

const Point stdi::operator*( const Point &p, int c )
{ return Point(p.xp*c, p.yp*c); }

const Point stdi::operator*( int c, const Point &p )
{ return Point(p.xp*c, p.yp*c); }

const Point stdi::operator*( const Point &p, double c )
{ return Point((Point::coord_type)(p.xp*c), (Point::coord_type)(p.yp*c)); }

const Point stdi::operator*( double c, const Point &p )
{ return Point((Point::coord_type)(p.xp*c), (Point::coord_type)(p.yp*c)); }


bool stdi::operator==( const Point &p1, const Point &p2 )
{ return p1.xp == p2.xp && p1.yp == p2.yp; }

bool stdi::operator!=( const Point &p1, const Point &p2 )
{ return p1.xp != p2.xp || p1.yp != p2.yp; }

const Point stdi::operator+( const Point &p1, const Point &p2 )
{ return Point(p1.xp+p2.xp, p1.yp+p2.yp); }

const Point stdi::operator-( const Point &p1, const Point &p2 )
{ return Point(p1.xp-p2.xp, p1.yp-p2.yp); }


const Point stdi::operator-( const Point &p )
{ return Point(-p.xp, -p.yp); }

Point &Point::operator/=( int c )
{
#if defined(CHECK_MATH)
  if ( c == 0 )
    warningDivByZero();
#endif
  xp/=(coord_type)c;
  yp/=(coord_type)c;
  return *this;
}

Point &Point::operator/=( double c )
{
#if defined(CHECK_MATH)
  if ( c == 0.0 )
    warningDivByZero();
#endif
  xp=(coord_type)(xp/c);
  yp=(coord_type)(yp/c);
  return *this;
}

const Point stdi::operator/( const Point &p, int c )
{
#if defined(CHECK_MATH)
  if ( c == 0 )
    warningDivByZero();
#endif
  return Point(p.xp/c, p.yp/c);
}

const Point stdi::operator/( const Point &p, double c )
{
#if defined(CHECK_MATH)
  if ( c == 0.0 )
    warningDivByZero();
#endif
  return Point((Point::coord_type)(p.xp/c), (Point::coord_type)(p.yp/c));
}


bool stdi::operator<( const Point &p1, const Point &p2 )
{ 
  if (p1.xp < p2.xp) return true;
  else  if (p1.xp > p2.xp) return false;
  if (p1.yp < p2.yp) return true;
  else return false;
}

/*****************************************************************************
  Point3D functions
  *****************************************************************************/

Point3D &Point3D::operator+=( const Point3D &p )
{ xp+=p.xp; yp+=p.yp; zp+=p.zp; return *this; }

Point3D &Point3D::operator-=( const Point3D &p )
{ xp-=p.xp; yp-=p.yp; zp-=p.zp; return *this; }

Point3D &Point3D::operator*=( int c )
{ xp*=(coord_type)c; yp*=(coord_type)c; zp*=(coord_type)c; return *this; }

Point3D &Point3D::operator*=( double c )
{ xp=(coord_type)(xp*c); yp=(coord_type)(yp*c); zp=(coord_type)(zp*c);return *this; }

bool stdi::operator==( const Point3D &p1, const Point3D &p2 )
{ return p1.xp == p2.xp && p1.yp == p2.yp && p1.zp == p2.zp ; }

bool stdi::operator!=( const Point3D &p1, const Point3D &p2 )
{ return p1.xp != p2.xp || p1.yp != p2.yp|| p1.zp != p2.zp ; }

const Point3D stdi::operator+( const Point3D &p1, const Point3D &p2 )
{ return Point3D(p1.xp+p2.xp, p1.yp+p2.yp, p1.zp+p2.zp); }

const Point3D stdi::operator-( const Point3D &p1, const Point3D &p2 )
{ return Point3D(p1.xp-p2.xp, p1.yp-p2.yp, p1.zp-p2.zp); }

const Point3D stdi::operator*( const Point3D &p, int c )
{ return Point3D(p.xp*c, p.yp*c, p.zp*c); }

const Point3D stdi::operator*( int c, const Point3D &p )
{ return Point3D(p.xp*c, p.yp*c, p.zp*c); }

const Point3D stdi::operator*( const Point3D &p, double c )
{ return Point3D( (Point3D::coord_type)(p.xp*c), 
		  (Point3D::coord_type)(p.yp*c), 
		  (Point3D::coord_type)(p.zp*c) ); }

const Point3D stdi::operator*( double c, const Point3D &p )
{ return Point3D((Point3D::coord_type)(p.xp*c), 
		 (Point3D::coord_type)(p.yp*c), 
		 (Point3D::coord_type)(p.zp*c)); 
}

const Point3D stdi::operator-( const Point3D &p )
{ return Point3D(-p.xp, -p.yp, -p.zp); }

Point3D &Point3D::operator/=( int c )
{
#if defined(CHECK_MATH)
  if ( c == 0 )
    warningDivByZero();
#endif
  xp/=(Point3D::coord_type)c;
  yp/=(Point3D::coord_type)c;
  zp/=(Point3D::coord_type)c;
  return *this;
}

Point3D &Point3D::operator/=( double c )
{
#if defined(CHECK_MATH)
  if ( c == 0.0 )
    warningDivByZero();
#endif
  xp=(coord_type)(xp/c);
  yp=(coord_type)(yp/c);
  zp=(coord_type)(zp/c);
  return *this;
}

const Point3D stdi::operator/( const Point3D &p, int c )
{
#if defined(CHECK_MATH)
  if ( c == 0 )
    warningDivByZero();
#endif
  return Point3D(p.xp/c, p.yp/c, p.zp/c);
}

const Point3D stdi::operator/( const Point3D &p, double c )
{
#if defined(CHECK_MATH)
  if ( c == 0.0 )
    warningDivByZero();
#endif
  return Point3D((Point3D::coord_type)(p.xp/c), (Point3D::coord_type)(p.yp/c), (Point3D::coord_type)(p.zp/c));
}
	
	
